@@ -20,9 +20,6 @@
 
 #define _GNU_SOURCE /* for getsid() */
 #include "mozplugger.h"
-#ifndef __sun
-#include <stdint.h>
-#endif
 
 static Display * display = 0;
 static int pipe_fd;
@@ -33,14 +30,31 @@ static char *command;
 static char *file;
 
 static XWindowAttributes wattr;
-static Window victim;
-static Window old_parent;
-static NPWindow windata;
+
+static struct
+{
+     int    mapped;
+     int    reparented;
+     Window window;
+     int    borderWidth;
+     int    x;
+     int    y;
+     int    width;
+     int    height;
+} victimDetails;
+
+static struct 
+{
+    Window  window;
+    int     width;
+    int     height;
+} parentDetails;
 
 static Atom swallowMutex;
 static int  swallowMutexTaken;
+static Atom windowOwnerMark;
 
-#define WINDOW ((Window) windata.window)
+#define WINDOW (parentDetails.window)
 
 int error_handler(Display *dpy, XErrorEvent *err)
 {
@@ -64,11 +78,14 @@ static void initSwallowMutex()
 	fprintf(stderr, "Display not set so cannot initialise semaphore!\n");
         return;
     }
+
     swallowMutex = XInternAtom(display, "MOZPLUGGER_SWALLOW_MUTEX", 0);
     swallowMutexTaken = 0;
+
+    windowOwnerMark = XInternAtom(display, "MOZPLUGGER_OWNER", 0);
 }
 
-uint32_t getHostId()
+uint32 getHostId()
 {
     char hostName[128];
     uint32 id;
@@ -84,14 +101,14 @@ uint32_t getHostId()
        effecting the swallow of victims is so infinitimisally small! */
 
     id = 0;
-    for(i=0; i < (int)(sizeof(hostName)/sizeof(uint32_t)); i++)
+    for(i=0; i < (int)(sizeof(hostName)/sizeof(uint32)); i++)
     {
-        id = ((id << 5) ^ (id >> 27)) ^ ((uint32_t *)hostName)[i];
+        id = ((id << 5) ^ (id >> 27)) ^ ((uint32 *)hostName)[i];
     }
     return id;
 }
 
-static int getSwallowMutexOwner(uint32_t * hostId, uint32_t * pid)
+static int getSwallowMutexOwner(uint32 * hostId, uint32 * pid)
 {   
     unsigned long nitems;
     unsigned long bytes;      
@@ -101,7 +118,9 @@ static int getSwallowMutexOwner(uint32_t * hostId, uint32_t * pid)
     int success = 0;
 
     /* Get hold of the Host & PID that current holds the semaphore for 
-       this display! */
+       this display! - watch out for the bizarre 64-bit platform behaviour
+       where property returned is actually an array of 2 x  64bit ints with
+       the top 32bits set to zero! */
     XGetWindowProperty(display, wattr.root, swallowMutex,
                        0, 2, 0, XA_INTEGER,
                        &type, &fmt, &nitems, &bytes,
@@ -110,7 +129,7 @@ static int getSwallowMutexOwner(uint32_t * hostId, uint32_t * pid)
     if(property)
     {
         /* Just check all is correct! */
-	if((type != XA_INTEGER) || (fmt!=32) || (nitems!=2) || (bytes != 0))
+	if((type != XA_INTEGER) || (fmt!=32) || (nitems!=2))
         {
 	    fprintf(stderr, "XGetWindowProperty returned bad values " 
                             "%ld,%d,%lu,%lu\n", (long) type, fmt, nitems, 
@@ -118,8 +137,8 @@ static int getSwallowMutexOwner(uint32_t * hostId, uint32_t * pid)
         }
 	else
 	{
-            *hostId = (uint32_t)((unsigned long *)property)[0];
-            *pid = (uint32_t)((unsigned long *)property)[1];
+            *hostId = (uint32)((unsigned long *)property)[0];
+            *pid = (uint32)((unsigned long *)property)[1];
             success = 1;
         }
 	XFree(property);
@@ -127,36 +146,36 @@ static int getSwallowMutexOwner(uint32_t * hostId, uint32_t * pid)
     return success;
 }
 
-static void setSwallowMutexOwner(uint32_t hostId, uint32_t pid)
+static void setSwallowMutexOwner(uint32 hostId, uint32 pid)
 {
     unsigned long temp[2] = {hostId, pid};
     D("Setting swallow mutex owner, hostId = 0x%08X, pid=%d\n", hostId, pid); 
+
     XChangeProperty(display, wattr.root, swallowMutex,
-                        XA_INTEGER, 32, PropModeReplace,
+                        XA_INTEGER, 32, PropModeAppend,
                         (unsigned char*) (&temp), 2);
-    XSync(display, False);
 }
- 
+
 static void takeSwallowMutex()
 {
     int countDown;
-    uint32_t ourPid = (uint32_t)getpid();
-    uint32_t ourHostId = getHostId();
-    uint32_t otherPid;
-    uint32_t otherHostId;
-    uint32_t prevOtherPid = 0;
+    uint32 ourPid = (uint32)getpid();
+    uint32 ourHostId = getHostId();
+    uint32 otherPid;
+    uint32 otherHostId;
+    uint32 prevOtherPid = 0;
  
-    D("Taking Swallow Mutex\n"); 
-
     if((display == 0) || (swallowMutex == 0))
 	return;
 
+    D("Attempting to take Swallow Mutex\n"); 
     /* Try up tp forty times ( 40 * 250ms = 10 seconds) to take 
        the semaphore... */
 
     countDown = 40;
     while(1)
     {
+
         /* While someone owns the semaphore */
         while(getSwallowMutexOwner(&otherHostId, &otherPid))
         {
@@ -168,6 +187,7 @@ static void takeSwallowMutex()
                        OR (unlikely) we previously had the semaphore! Exit
                        the function...*/
 	            swallowMutexTaken = 1;
+                    D("Taken Swallow Mutex\n"); 
 		    return;
                 }
 
@@ -177,8 +197,8 @@ static void takeSwallowMutex()
                    if process exists. */
             	if( (getsid(otherPid) < 0 ) && (errno == ESRCH))
 	        {
-	    	    fprintf(stderr,"Strange other Pid(%lu) cannot be found\n", 
-                                   otherPid);
+	    	    D("Strange other Pid(%lu) cannot be found\n", otherPid);
+                    XDeleteProperty(display, wattr.root, swallowMutex);
                     break;    /* OK force early exit of inner while loop */
                 }
 	    }
@@ -187,8 +207,8 @@ static void takeSwallowMutex()
                has restart timer */
             if(prevOtherPid != otherPid)
             {
-     	        D("Looks like semaphore's owner has changed pid=%ld\n", 
-                   otherPid);
+   	        D("Looks like semaphore's owner has changed pid=%ld\n", 
+                     otherPid);
 	        countDown = 40;
                 prevOtherPid = otherPid;
             }
@@ -196,7 +216,11 @@ static void takeSwallowMutex()
             /* Do one step of the timer... */
 	    countDown--;
             if(countDown <= 0)
+            {
+	    	D("Waited long enough for Pid(%lu)\n", otherPid);
+                XDeleteProperty(display, wattr.root, swallowMutex);
 		break;
+            }
 	
             usleep(250000);        /* 250ms */
  	}  
@@ -210,14 +234,93 @@ static void takeSwallowMutex()
 
 static void giveSwallowMutex()
 {
-    D("Giving Swallow Mutex\n"); 
     if((display == 0) || (swallowMutex == 0) || (swallowMutexTaken ==0))
 	return;
 
+    D("Giving Swallow Mutex\n"); 
     XDeleteProperty(display, wattr.root, swallowMutex);
     swallowMutexTaken = 0;
 }
 
+/*****************************************************************************
+ * Mark the victim window with a property that indicates that this instance
+ * of mozplugger-helper has made a claim to this victim. This is used to 
+ * guard against two instances of mozplugger-helper claiming the same victim.
+ *****************************************************************************/
+static int chkAndMarkVictimWindow(Window w)
+{
+     unsigned long nitems;
+     unsigned long bytes;      
+     int fmt;
+     Atom type;
+     unsigned char * property = NULL;
+     unsigned long temp[2];
+     uint32 ourPid;
+     uint32 ourHostId;
+     int gotIt = 0;
+
+     /* See if some other instance of Mozplugger has already 'marked' this
+      * window */
+     XGetWindowProperty(display, w, windowOwnerMark,
+                        0, 2, 0, XA_INTEGER,
+                        &type, &fmt, &nitems, &bytes,
+                        &property);
+
+     if(property)
+     {
+          /* Just check all is correct! */
+	  if((type != XA_INTEGER) || (fmt!=32) || (nitems!=2))
+          {
+	       D("XGetWindowProperty(windowOwnerMark) returned bad values " 
+                                   "%ld,%d,%lu,%lu\n", (long) type, fmt, nitems, 
+                                   bytes);
+          }
+  	  XFree(property);
+          return 0;          /* Looks like someone else has marked this window */
+     }
+
+     /* OK lets claim it for ourselves... */
+     ourPid = (uint32)getpid();
+     ourHostId = getHostId();
+
+     temp[0] = ourHostId;
+     temp[1] = ourPid;
+
+     XChangeProperty(display, w, windowOwnerMark,
+                     XA_INTEGER, 32, PropModeAppend,
+                     (unsigned char*) (&temp), 2);
+
+     /* See if we got it */
+     XGetWindowProperty(display, w, windowOwnerMark,
+                        0, 2, 0, XA_INTEGER,
+                        &type, &fmt, &nitems, &bytes,
+                        &property);
+
+     if(property)
+     {
+
+          /* Just check all is correct! */
+          if((type != XA_INTEGER) || (fmt!=32) || (nitems!=2))
+          {
+               fprintf(stderr, "XGetWindowProperty returned bad values " 
+                                   "%ld,%d,%lu,%lu\n", (long) type, fmt, nitems, 
+                                   bytes);
+          }
+          else if( ( ((unsigned long *)property)[1] == ourPid)
+                  && ( ((unsigned long *)property)[0] == ourHostId) )
+          {
+               gotIt = 1;
+          }
+  	  XFree(property);
+     }
+     else         
+     {
+         D("Strange, couldn't set windowOwnerMark\n");
+         gotIt = 1;
+     }
+
+     return gotIt;
+}
 
 /*****************************************************************************
  * Window resizing
@@ -248,24 +351,28 @@ static void adjust_window_size(void)
 {
      int x = 0;
      int y = 0;
-     int w = (int) windata.width;
-     int h = (int) windata.height;
+     int w = parentDetails.width;
+     int h = parentDetails.height;
 
      int tmpw, tmph;
 
-     if (!victim) return;
-     if (!WINDOW) return;
-     if (old_parent != WINDOW) return;
+     if (!victimDetails.window) return;
+     
+     if (!victimDetails.mapped || !victimDetails.reparented)
+     {
+          D("Victim not ready to be resized\n");
+          return;
+     }
 
      if (flags & H_FILL)
      {
-	  D("Resizing window %x with FILL\n", victim);
+	  D("Resizing window %x with FILL\n", victimDetails.window);
      }
      else if (flags & H_MAXASPECT)
      {
 	  if (xaspect && yaspect)
 	  {
-	       D("Resizing window %x with MAXASPECT\n", victim);
+	       D("Resizing window %x with MAXASPECT\n", victimDetails.window);
 	       tmph = h / yaspect;
 	       tmpw = w / xaspect;
 	       if (tmpw < tmph) tmph = tmpw;
@@ -285,9 +392,46 @@ static void adjust_window_size(void)
 	  }
      }
   
+     /* Compensate for the Victims border width (usually set to zero anyway) */
+     w -= 2 * victimDetails.borderWidth;
+     h -= 2 * victimDetails.borderWidth;
+
      D("New size: %dx%d+%d+%d\n", w, h, x, y);
-     
-     XMoveResizeWindow(display, victim, x, y, (unsigned)w, (unsigned)h);
+
+     if((victimDetails.x == x) && (victimDetails.y == y)
+        && (victimDetails.width == w) && (victimDetails.height == h))
+     {
+          XEvent event;
+          D("No change in window size so sending ConfigureNotify instead\n");
+          /* According to X11 ICCCM specification, a compliant  window 
+           * manager, (or proxy in this case) should sent a  ConfigureNotify
+           * event to the client even if no size change has occurred!
+           * The code below is taken and adapted from the
+           * TWM window manager and fixed movdev bug #18298. */
+          event.type = ConfigureNotify;
+          event.xconfigure.display = display;
+          event.xconfigure.event = victimDetails.window;
+          event.xconfigure.window = victimDetails.window;
+          event.xconfigure.x = x;
+          event.xconfigure.y = y;
+          event.xconfigure.width = w;
+          event.xconfigure.height = h;
+          event.xconfigure.border_width = victimDetails.borderWidth;
+          event.xconfigure.above = Above;
+          event.xconfigure.override_redirect = False;
+
+          XSendEvent(display, victimDetails.window, False, StructureNotifyMask,
+                  &event);
+     }
+     else
+     {
+          XMoveResizeWindow(display, victimDetails.window, 
+                                     x, y, (unsigned)w, (unsigned)h);
+          victimDetails.x = x;
+          victimDetails.y = y;
+          victimDetails.width = w;
+          victimDetails.height = h;
+    }
 }
 
 /*****************************************************************************
@@ -297,29 +441,28 @@ static void adjust_window_size(void)
 static void change_leader()
 {
      XWMHints *leader_change;
-     if ((leader_change = XGetWMHints(display, victim)))
+     if ((leader_change = XGetWMHints(display, victimDetails.window)))
      {
 	  leader_change->flags = (leader_change->flags | WindowGroupHint);
 	  leader_change->window_group = wattr.root;
-	  XSetWMHints(display,victim,leader_change);
+	  XSetWMHints(display,victimDetails.window,leader_change);
 	  XFree(leader_change);
      }
 }
 
 static int reparent_window(void)
 {
-     if (!victim) return 0;
+     if (!victimDetails.window) return 0;
      if (!WINDOW) return 0;
-     if (old_parent == WINDOW) return 0;
 
-     XSelectInput(display, victim, StructureNotifyMask); 
+     XSelectInput(display, victimDetails.window, StructureNotifyMask); 
      XSync(display, False);
 
-     D("Changing leader of window %x\n", victim);
+     D("Changing leader of window %x\n", victimDetails.window);
      change_leader();
      
-     D("Reparenting window %x into %x\n", victim, WINDOW);
-     XReparentWindow(display, victim, WINDOW, 0, 0);
+     D("Reparenting window %x into %x\n", victimDetails.window, WINDOW);
+     XReparentWindow(display, victimDetails.window, WINDOW, 0, 0);
 
      D("reparent_window() done\n");
 
@@ -366,7 +509,7 @@ static int my_strcmp(char *windowname, char *name)
      }
 }
 
-static char check_window_name(Window w, char *name, int (*cb)(Window))
+static char check_window_name(Window w, char *name)
 {
      char *windowname;
      XClassHint windowclass;
@@ -380,7 +523,7 @@ static char check_window_name(Window w, char *name, int (*cb)(Window))
 	  match = (my_strcmp(windowname, name) == 0);
 	  XFree(windowname);
 	  if (match) {
-	       return cb(w);
+	       return 1;
 	  }
      }
      
@@ -392,80 +535,134 @@ static char check_window_name(Window w, char *name, int (*cb)(Window))
 	  match = (my_strcmp(windowclass.res_name, name) == 0);
 	  XFree(windowclass.res_class);
 	  XFree(windowclass.res_name);
-	  if (match) {
-	       return cb(w);
-	  }
+          return match;
      }
      return 0;
 }
 
-static Window winrecur(Window from, int depth, char *name, int (*cb)(Window))
+#define MAX_IGNORED 3000
+static unsigned int ignore_window_count = 0;
+static Window ignored[MAX_IGNORED];
+
+/*****************************************************************************
+ * init_winrecur
+ *
+ * This function should be called before forking the application. It runs 
+ * through all windows on the screen to a depth of 'depth' and records their
+ * identify in the ignored list. 
+ *****************************************************************************/
+static void init_winrecur(Window from, int depth, char * name)
 {
      Window root, parent;
      Window *children;
      unsigned int e;
      unsigned int num_children;
 
-     D("Winrecur, checking window %x (depth=%d)\n",from,depth);
+     D("init_winrecur: storing family tree of window %x (depth=%d)\n", 
+                                                          from, depth);
      
-     if (!(flags & H_HIDDEN) && !is_viewable(display, from))
-	  return (Window)0;
-   
-     if (check_window_name(from, name, cb))
-	  return from;
-
-     if (!XQueryTree(display, from, &root, &parent, &children, &num_children))
+     if (check_window_name(from, name))
      {
-	  D("Querytree failed!!!\n");
-	  return (Window)0;
+          if (ignore_window_count < MAX_IGNORED) 
+          {
+               ignored[ignore_window_count++] = from;
+          }
+          else
+          {
+               D("Woo too many windows!\n");
+          }
+          return;
      }
 
      if (depth > 0)
      {
-	  D("Num children = %d\n", num_children);
+          if (!XQueryTree(display, from, &root, &parent, 
+                                         &children, &num_children))
+          {
+	       D("init_winrecur: Querytree failed!!!\n");
+	       return;
+          }
+
+	  D("init_winrecur: Num children = %d\n", num_children);
 	  for (e = 0; e < num_children; e++)
 	  {
-	       Window win = winrecur(children[e], depth-1, name, cb);
+	       init_winrecur(children[e], depth-1, name);
+	  }
+     }
+     XFree(children);
+}
+
+/*****************************************************************************
+ * find_winrecur
+ *
+ * This function should be called after forking the application and tries to
+ * see if this window (or its child) are the ones to swallow. 
+ *****************************************************************************/
+static Window find_winrecur(Window from, int depth, char *name)
+{
+     Window root, parent;
+     Window *children;
+     unsigned int e;
+     unsigned int num_children;
+
+     D("find_winrecur, checking window %x (depth=%d)\n",from,depth);
+     
+     for (e=0; e < ignore_window_count; e++) 
+     {
+	  if (ignored[e] == from)
+          {
+               /* This window is OLD, ignore it (and its children) */
+	       return (Window)0;
+          }
+     }
+
+     /* This window doesn't match our 'HIDDEN' criteria, ignore it
+      * (and its children) */
+     if (!(flags & H_HIDDEN) && !is_viewable(display, from))
+	  return (Window)0;
+  
+     if (check_window_name(from, name))
+     {
+          /* See if some instance of mozplugger got the window before us, if
+           * not mark it as ours */
+          if(chkAndMarkVictimWindow(from))
+          {
+	        return from;
+          }
+     }
+
+     if (depth > 0)
+     {
+          /* OK if we get here, lets check out the children just in case the
+           * application has decided to name a child not the parent?
+           * Question? Does this ever happen? */
+
+          if (!XQueryTree(display, from, &root, &parent, 
+                                         &children, &num_children))
+          {
+	        D("find_winrecur: Querytree failed!!!\n");
+	        return (Window)0;
+          }
+
+	  D("find_winrecur: Num children = %d\n", num_children);
+	  for (e = 0; e < num_children; e++)
+	  {
+	       Window win = find_winrecur(children[e], depth-1, name);
 	       if (win)
 	       {
 		    XFree(children);
 		    return (Window)win;
 	       }
 	  }
+          XFree(children);
      }
-
-     XFree(children);
      return (Window)0;
 }
 
-#define MAX_IGNORED 3000
-static int windows_found = 0;
-static Window ignored[MAX_IGNORED];
-
-int initial_win_cb(Window w)
-{
-     if (windows_found < MAX_IGNORED) {
-	  ignored[windows_found++] = w;
-	  D("Ignoring window %x\n", w);
-     }
-     return 0;
-}
-
-/* Returns 1 if and only if w is a new window. */
-int win_cb(Window w)
-{
-     int i;
-     for (i=0; i < windows_found; i++) {
-	  if (ignored[i] == w)
-	       return 0;
-     }
-     return 1;
-}
-
 void save_initial_windows()
 {
-     windows_found = 0;
-     winrecur(wattr.root, 3, winname, initial_win_cb);
+     ignore_window_count = 0;
+     init_winrecur(wattr.root, 3, winname);
 }
 
 static int setup_display()
@@ -557,15 +754,23 @@ static void run_app(char **argv)
  *****************************************************************************/
 static void find_victim(Window window)
 {
-     if (!victim)
+     if (!victimDetails.window)
      {
 	  D("Looking for victim... (%s)\n", winname);
 	  
-	  victim = winrecur(window, 2, winname, win_cb);
-
-	  if (victim)
+	  victimDetails.window = find_winrecur(window, 2, winname);
+          
+	  if (victimDetails.window)
 	  {
-	       D("Found it!! Victim=%x\n", victim);
+	       XWindowAttributes ca;
+	       XGetWindowAttributes(display, victimDetails.window, &ca);
+
+   	       D("Found it!! Victim=%x\n", victimDetails.window);
+               victimDetails.borderWidth = ca.border_width;
+               victimDetails.x = ca.x;
+               victimDetails.y = ca.y;
+               victimDetails.width = ca.width;
+               victimDetails.height = ca.height;
 
 	       /* We don't need to listen for new windows anymore */
 	       XSelectInput(display, wattr.root, 0);
@@ -573,12 +778,8 @@ static void find_victim(Window window)
  	
 	       if (flags & H_MAXASPECT)
 	       {
-		    XWindowAttributes ca;
-		    XGetWindowAttributes(display, victim, &ca);
 		    set_aspect(ca.width, ca.height);
 	       }
-
-	       old_parent = 0;
 	       reparent_window();
 	  }
      }
@@ -592,84 +793,132 @@ static Bool AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
 static void check_x_events(void)
 {
      XEvent ev;
-     static char mapped = 0;
-     static char reparented = 0;
 
      while (XCheckIfEvent(display, &ev, AllXEventsPredicate, NULL))
      {
-	  D("got event %s->%d\n",
-	    ev.xany.window == wattr.root ? "root" :
-	    ev.xany.window == victim ? "victim" : 
-	    ev.xany.window == WINDOW ? "WINDOW" : 
-	    "unknown", ev.type);
-
 	  if (ev.xany.window == wattr.root)
 	  {
 	       switch (ev.type)
 	       {
 	       case CreateNotify:
-		    D("***CreateNotify\n");
+		    D("***CreateNotify for root\n");
 		    find_victim(ev.xcreatewindow.window);
 		    break;
 
 	       case MapNotify:
-		    D("***MapNotify: %x\n", ev.xmap.window);
+		    D("***MapNotify for root: %x\n", ev.xmap.window);
 		    find_victim(ev.xmap.window);
 		    break;
 
 	       case ConfigureNotify:
-		    D("***ConfigureNotify: %x\n", ev.xconfigure.window);
+		    D("***ConfigureNotify for root: %x\n", ev.xconfigure.window);
 		    find_victim(ev.xconfigure.window);
 		    break;
+
+               default: 
+                   /* 21 = ReparentNotify */
+	           D("!!Got unhandled event for root->%d\n", ev.type);
+                   break;
 	       }
 	  }
-	  else if (victim && ev.xany.window == victim)
+	  else if (victimDetails.window 
+                               && ev.xany.window == victimDetails.window)
 	  {
 	       switch (ev.type)
 	       {
 	       case UnmapNotify:
-		    D("UNMAPNOTIFY\n");
-		    mapped = 0;
+		    D("UNMAPNOTIFY for victim\n");
+		    victimDetails.mapped = 0;
 		    break;
 		    
 	       case MapNotify:
-		    D("MAPNOTIFY\n");
-		    mapped = 1;
+		    D("MAPNOTIFY for victim\n");
+		    victimDetails.mapped = 1;
 		    adjust_window_size();
 		    break;
 	    
 	       case ReparentNotify:
-		    old_parent = ev.xreparent.parent;
 		    if (ev.xreparent.parent == WINDOW)
 		    {
-			 D("REPARENT NOTIFY to the right window\n");
+			 D("REPARENT NOTIFY on victim to the right window\n");
 			 adjust_window_size();
-			 reparented = 1;
+			 victimDetails.reparented = 1;
 		    } else {
-			 D("REPARENT NOTIFY to some other window!\n");
+			 D("REPARENT NOTIFY on victim to some other window!\n");
 			 reparent_window();
-			 reparented = 0;
+			 victimDetails.reparented = 0;
 		    }
 		    break;
+
+	       case ConfigureNotify:
+		    D("CONFIGURE NOTIFY for victim\n");
+		    D("- x=%d, y=%d, w=%d, h=%d, border_width=%d\n",
+                          ev.xconfigure.x, ev.xconfigure.y, 
+                          ev.xconfigure.width, ev.xconfigure.height, 
+                          ev.xconfigure.border_width);
+		    break;
+
+               case ClientMessage:
+		    D("CLIENT MESSAGE for victim\n");
+                    /* I see this with evince! perhaps it needs to be handled? */
+                    break;
+
+               default: 
+                   /* 22 = ConfigureNotify, 33 = ClientMessage */
+	           D("!!Got unhandled event for victim->%d\n", ev.type);
+                   break;
 	       }
 	  }
 	  else if (WINDOW && ev.xany.window == WINDOW)
 	  {
+               unsigned long mask;
 	       switch (ev.type)
 	       {
 	       case ConfigureRequest:
-		    D("ConfigureRequest\n");
-		    set_aspect(ev.xconfigure.width, ev.xconfigure.height);
-		    adjust_window_size();
+                    mask = ev.xconfigurerequest.value_mask;
+		    D("ConfigureRequest to WINDOW mask=0x%x\n", mask);
+                    if(ev.xconfigurerequest.window != victimDetails.window)
+                    {
+                         D("- Strange Configure Request not for victim\n");
+                         break;
+                    }
+                    if(mask & (CWHeight | CWWidth))
+                    {
+                         D(" - request to set width & height\n");
+		         set_aspect(ev.xconfigurerequest.width, 
+                                     ev.xconfigurerequest.height);
+		         adjust_window_size();
+                    }
+                    if(mask & (CWBorderWidth))
+                    {
+                         const int w =  ev.xconfigurerequest.border_width; 
+                         D("- request to set border width=%d\n", w);
+                         if(victimDetails.borderWidth != w)
+                         {
+                              victimDetails.borderWidth = w;
+                              adjust_window_size();
+                         }
+                         XSetWindowBorderWidth(display, victimDetails.window, 
+                                                        (unsigned) w);
+                    }
 		    break;
+
+               default: 
+	           D("!!Got unhandled event for WINDOW->%d\n", ev.type);
+                   break;
 	       }
 	  }
+          else
+          {
+	       D("!!Got unhandled event for uknown->%d\n", ev.type);
+          }
      }
 
      /* For Window Maker */
-     if (reparented && !mapped) {
+     if (victimDetails.reparented && !victimDetails.mapped) 
+     {
 	  D("XMapWindow\n");
-	  XMapWindow(display, victim);
+	  XMapWindow(display, victimDetails.window);
      }
 }
 
@@ -697,24 +946,26 @@ static void check_pipe_fd_events(void)
 	  exit(EX_UNAVAILABLE);
      }
   
-     if (n != sizeof(windata))
+     if (n != sizeof(wintmp))
 	  return;
   
-     windata = wintmp;
-  
+     parentDetails.window = (Window) wintmp.window;
+     parentDetails.width = (int) wintmp.width;
+     parentDetails.height = (int) wintmp.height;
+
      D("Got pipe_fd data, new parent=%x\n", WINDOW);
   
      if (WINDOW && WINDOW != oldwindow && display)
      {
+          victimDetails.reparented = 0;
+	  XSelectInput(display, oldwindow, 0);
 	  XSelectInput(display, WINDOW, SubstructureRedirectMask);
 	  XSync(display, False);
+          reparent_window();
      }
   
-     if (victim)
-     {
-	  /* The window has been resized. */
-	  adjust_window_size();
-     }
+     /* The window has been resized. */
+     adjust_window_size();
 }
 
 static void check_all_events()
@@ -732,7 +983,10 @@ static void check_all_events()
      tv.tv_sec = SELECT_TIMEOUT_SEC;
      tv.tv_usec = SELECT_TIMEOUT_USEC;
      
+#if 0
      D("SELECT IN\n");
+#endif
+
      if (select(maxfd + 1, &fds, NULL, NULL, &tv) > 0)
      {
 	  D("SELECT OUT\n");
@@ -794,6 +1048,7 @@ static void handle_app(pid_t pid)
 int main(int argc, char **argv)
 {
      unsigned long temp = 0;
+     int x, y;
      D("Helper started.....\n");
 
      if (argc < 2)
@@ -803,17 +1058,20 @@ int main(int argc, char **argv)
 	  exit(1);
      }
 
+     memset(&victimDetails, 0, sizeof(victimDetails));
+     memset(&parentDetails, 0, sizeof(parentDetails));
+
      sscanf(argv[1],"%d,%d,%d,%lu,%d,%d,%d,%d",
 	    &flags,
 	    &repeats,
 	    &pipe_fd,
 	    &temp,
-	    (int *)&windata.x,
-	    (int *)&windata.y,
-	    (int *)&windata.width,
-	    (int *)&windata.height);
+	    &x,
+	    &y,
+	    &parentDetails.width,
+	    &parentDetails.height);
 
-     windata.window = (void *)temp;
+     parentDetails.window = (Window)temp;
 
      command = argv[2];
      winname = getenv("winname");
